// Spatial helpers functions for use in bob
// Useful for 3d things, etc
// Especially in the ugly as fuck graphics code

#ifndef _SPATIAL_HELPERS_H_
#define _SPATIAL_HELPERS_H_

#include <armadillo>

// Axis angle functions
void create_axisangle(double *theta, double *eaxis, double *a, double *b);
void axisangle_from_quaternion(double *theta, double *eaxis, double *q);

// quaternion functions
void quaternion_from_axisangle(double *q, double theta, double *eaxis);
void quaternion_between_vectors(double *q, double *a, double *b);
void quaternion_multiply(double *t, double *q, double *r);
double quaternion_norm(double *q);
void normalize_quaternion(double *q);

// Rotation functions
void rodrigues_axisangle(double *vrot, double *v, double theta, double *e);
void rotate_towards(double *q1, double *q2, float maxAngle, double *q);

// Print functionality
void print_quaternion(double *q);

// New quaternion formulation, using the armadillo C++ library
inline void create_rotation_matrix_body_space(arma::mat33& A,
                                              double *u,
                                              double *v,
                                              double *w) {
    arma::vec3 u_ = {u[0], u[1], u[2]};
    arma::vec3 v_ = {v[0], v[1], v[2]};
    arma::vec3 w_ = {w[0], w[1], w[2]};
    A.col(0) = u_;
    A.col(1) = v_;
    A.col(2) = w_;
}

/*! \brief Create a quaternion from a rotation matrix
 *
 * Creates a quaternion from a rotation matrix, and uses the 'best' set of
 * coodinates and calculations to avoid floating point error as much as
 * possible. This is using Shepperd's method (Terzakis 2012).
 */
inline void quaternion_from_rotation_matrix(arma::vec4& q,
                                            const arma::mat33 R) {
    if ((R(1,1) >= -R(2,2)) && (R(0,0) >= -R(1,1)) && (R(0,0) >= -R(2,2))) {
        q[0] = std::sqrt(1 + R(0,0) + R(1,1) + R(2,2));
        q[1] = (R(2,1) - R(1,2)) / std::sqrt(1 + R(0,0) + R(1,1) + R(2,2));
        q[2] = (R(0,2) - R(2,0)) / std::sqrt(1 + R(0,0) + R(1,1) + R(2,2));
        q[3] = (R(1,0) - R(0,1)) / std::sqrt(1 + R(0,0) + R(1,1) + R(2,2));
    } else if ((R(1,1) <= -R(2,2)) && (R(0,0) >=  R(1,1)) && (R(0,0) >=  R(2,2))) {
        q[0] = (R(2,1) - R(1,2)) / std::sqrt(1 + R(0,0) - R(1,1) - R(2,2));
        q[1] = std::sqrt(1 + R(0,0) - R(1,1) - R(2,2));
        q[2] = (R(1,0) + R(0,1)) / std::sqrt(1 + R(0,0) - R(1,1) - R(2,2));
        q[3] = (R(2,0) + R(0,2)) / std::sqrt(1 + R(0,0) - R(1,1) - R(2,2));
    } else if ((R(1,1) >=  R(2,2)) && (R(0,0) <=  R(1,1)) && (R(0,0) <= -R(2,2))) {
        q[0] = (R(0,2) - R(2,0)) / std::sqrt(1 - R(0,0) + R(1,1) - R(2,2));
        q[1] = (R(1,0) + R(0,1)) / std::sqrt(1 - R(0,0) + R(1,1) - R(2,2));
        q[2] = std::sqrt(1 - R(0,0) + R(1,1) - R(2,2));
        q[0] = (R(2,1) + R(1,2)) / std::sqrt(1 - R(0,0) + R(1,1) - R(2,2));
    } else if ((R(1,1) <=  R(2,2)) && (R(0,0) <= -R(1,1)) && (R(0,0) <=  R(2,2))) {
        q[0] = (R(1,0) - R(0,1)) / std::sqrt(1 - R(0,0) - R(1,1) + R(2,2));
        q[1] = (R(2,0) + R(0,2)) / std::sqrt(1 - R(0,0) - R(1,1) + R(2,2));
        q[2] = (R(2,1) + R(1,2)) / std::sqrt(1 - R(0,0) - R(1,1) + R(2,2));
        q[3] = std::sqrt(1 - R(0,0) - R(1,1) + R(2,2));
    } else {
        std::cerr << "ERROR: quaterion_from_rotation_matrix did not work correctly!\n";
        exit(1);
    }

    // Have to multiply by 1/2 on all cases
    for (auto i = 0; i < 4; ++i) {
        q[i] = 0.5 * q[i];
    }
}

/*! \brief Create a rotation matrix from a quaternion
 *
 * Creates a rotation matrix from a quaternion describing orientation of a
 * rigid body (wikipedia: quaternions and spatial rotations)
 */
inline void rotation_matrix_from_quaternion(arma::mat33& R,
                                            const arma::vec4& q) {
    R(0,0) = 1. - 2.*(q[2] * q[2] + q[3] * q[3]);
    R(0,1) = 2. * (q[1] * q[2] - q[3] * q[0]);
    R(0,2) = 2. * (q[1] * q[3] + q[2] * q[0]);

    R(1,0) = 2. * (q[1] * q[2] + q[3] * q[0]);
    R(1,1) = 1 - 2. * (q[1] * q[1] + q[3] * q[3]);
    R(1,2) = 2. * (q[2] * q[3] - q[1] * q[0]);

    R(2,0) = 2. * (q[1] * q[3] - q[2] * q[0]);
    R(2,1) = 2. * (q[2] * q[3] + q[1] * q[0]);
    R(2,2) = 1 - 2. * (q[1] * q[1] + q[2] * q[2]);
}

#endif
